require 'thor'
require 'net/hippie'

module Ghetto
  module Blaster
    class CLI < Thor
      class_option :client_id, default: ENV['SOUNDCLOUD_CLIENT_ID']

      desc "play username", "play music by a specific user"
      def play(username)
        tracks_for(username).each do |track|
          stream_url = "#{track[:stream_url]}s/?client_id=#{client_id}"
          json = JSON.parse(client.get(stream_url).body, symbolize_names: true)
          say json[json.keys.first]
        end
      end

      desc "user username", "fetch user info"
      def user(username)
        say client.get(resolve(username)[:location]).body
      end

      desc "tracks username", "fetch tracks for user"
      def tracks(username)
        say tracks_for(username, parse: false)
      end

      desc "version", "print version"
      def version
        say Ghetto::Blaster::VERSION
      end

      private

      def tracks_for(username, parse: true)
        user_id = resolve(username)[:location].split('?')[0].split('/').last
        body = client.get(build_uri("/users/#{user_id}/tracks?client_id=#{client_id}")).body
        parse ? JSON.parse(body, symbolize_names: true) : body
      end

      def client
        @client ||= Net::Hippie::Client.new
      end

      def client_id
        options[:client_id]
      end

      def build_uri(path)
        URI.join("https://api.soundcloud.com", path)
      end

      def resolve(username)
        JSON.parse(client.get(build_uri("/resolve?url=https://soundcloud.com/#{username}&client_id=#{client_id}")).body, symbolize_names: true)
      end
    end
  end
end
